@@ -5,13 +5,15 @@ use helix_event::AsyncHook;
 
 use crate::config::Config;
 use crate::events;
+use crate::handlers::auto_save::AutoSaveHandler;
 use crate::handlers::completion::CompletionHandler;
 use crate::handlers::signature_help::SignatureHelpHandler;
 
 pub use completion::trigger_auto_completion;
 pub use helix_view::handlers::Handlers;
 
-mod completion;
+mod auto_save;
+pub mod completion;
 mod signature_help;
 
 pub fn setup(config: Arc<ArcSwap<Config>>) -> Handlers {
@@ -19,11 +21,16 @@ pub fn setup(config: Arc<ArcSwap<Config>>) -> Handlers {
 
     let completions = CompletionHandler::new(config).spawn();
     let signature_hints = SignatureHelpHandler::new().spawn();
+    let auto_save = AutoSaveHandler::new().spawn();
+
     let handlers = Handlers {
         completions,
         signature_hints,
+        auto_save,
     };
+
     completion::register_hooks(&handlers);
     signature_help::register_hooks(&handlers);
+    auto_save::register_hooks(&handlers);
     handlers
 }
